Package: PlackettLuce
Type: Package
Title: Plackett-Luce Models for Rankings
Version: 0.4.5
Authors@R: c(person("Heather", "Turner",
        email = "ht@heatherturner.net", role = c("aut", "cre"),
        comment = c(ORCID = "0000-0002-1256-3375")),
        person("Ioannis", "Kosmidis", role = "aut",
        comment = c(ORCID = "0000-0003-1556-0302")),
        person("David", "Firth", role = "aut",
        comment = c(ORCID = "0000-0003-0302-2312")),
        person("Jacob", "van Etten", role = "ctb",
        comment = c(ORCID = "0000-0001-7554-2558")))
URL: https://hturner.github.io/PlackettLuce/,
        https://github.com/hturner/PlackettLuce
BugReports: https://github.com/hturner/PlackettLuce/issues
Description: Functions to prepare rankings data and fit the Plackett-Luce model
    jointly attributed to Plackett (1975) <doi:10.2307/2346567> and Luce
    (1959, ISBN:0486441369). The standard Plackett-Luce model is generalized
    to accommodate ties of any order in the ranking. Partial rankings, in which
    only a subset of items are ranked in each ranking, are also accommodated in
    the implementation. Disconnected/weakly connected networks implied by the
    rankings may be handled by adding pseudo-rankings with a hypothetical item.
    Optionally, a multivariate normal prior may be set on the log-worth
    parameters and ranker reliabilities may be incorporated as proposed by
    Raman and Joachims (2014) <doi:10.1145/2623330.2623654>. Maximum a
    posteriori estimation is used when priors are set. Methods are provided to
    estimate standard errors or quasi-standard errors for inference as well as
    to fit Plackett-Luce trees. See the package website or vignette for further
    details.
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.5)
Imports: CVXR, igraph, Matrix, matrixStats, methods, partykit,
        psychotools, psychotree, R6, RSpectra, qvcalc, sandwich, stats
Suggests: BayesMallows, BradleyTerry2, PLMIX, ROlogit, StatRank,
        bookdown, covr, hyper2, kableExtra, knitr, lbfgs, gnm, pmr,
        rmarkdown, survival, testthat
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Language: en-GB
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-03-24 11:14:15 UTC; h.l.turner@bham.ac.uk
Author: Heather Turner [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1256-3375>),
  Ioannis Kosmidis [aut] (ORCID: <https://orcid.org/0000-0003-1556-0302>),
  David Firth [aut] (ORCID: <https://orcid.org/0000-0003-0302-2312>),
  Jacob van Etten [ctb] (ORCID: <https://orcid.org/0000-0001-7554-2558>)
Maintainer: Heather Turner <ht@heatherturner.net>
Repository: CRAN
Date/Publication: 2026-03-25 10:20:03 UTC
Built: R 4.6.0; ; 2026-04-27 17:22:02 UTC; unix
