# Generated by roxygen2: do not edit by hand

S3method("[",aggregated_rankings)
S3method("[",grouped_rankings)
S3method("[",rankings)
S3method(AIC,pltree)
S3method(aggregate,aggregated_rankings)
S3method(aggregate,rankings)
S3method(anova,PLADMM)
S3method(anova,PLADMMlist)
S3method(as.aggregated_rankings,preflib)
S3method(as.data.frame,grouped_rankings)
S3method(as.data.frame,rankings)
S3method(as.grouped_rankings,paircomp)
S3method(as.matrix,aggregated_rankings)
S3method(as.matrix,rankings)
S3method(as.rankings,default)
S3method(as.rankings,grouped_rankings)
S3method(as.rankings,matrix)
S3method(coef,PlackettLuce)
S3method(coef,pltree)
S3method(deviance,PLADMM)
S3method(deviance,PlackettLuce)
S3method(estfun,PLADMM)
S3method(estfun,PlackettLuce)
S3method(fitted,PLADMM)
S3method(fitted,PlackettLuce)
S3method(fitted,pltree)
S3method(format,grouped_rankings)
S3method(format,rankings)
S3method(group,rankings)
S3method(is.na,grouped_rankings)
S3method(is.na,rankings)
S3method(itempar,PLADMM)
S3method(itempar,PlackettLuce)
S3method(itempar,pltree)
S3method(length,rankings)
S3method(logLik,PLADMM)
S3method(logLik,PlackettLuce)
S3method(model.frame,pltree)
S3method(plot,pltree)
S3method(predict,PLADMM)
S3method(predict,pltree)
S3method(print,PLADMM)
S3method(print,PlackettLuce)
S3method(print,choices)
S3method(print,coef.PlackettLuce)
S3method(print,grouped_rankings)
S3method(print,pltree)
S3method(print,rankings)
S3method(print,summary.PLADMM)
S3method(print,summary.PlackettLuce)
S3method(qvcalc,PlackettLuce)
S3method(rbind,aggregated_rankings)
S3method(rbind,rankings)
S3method(simulate,PlackettLuce)
S3method(summary,PLADMM)
S3method(summary,PlackettLuce)
S3method(vcov,PLADMM)
S3method(vcov,PlackettLuce)
S3method(vcov,pltree)
export(PlackettLuce)
export(adjacency)
export(as.aggregated_rankings)
export(as.grouped_rankings)
export(as.rankings)
export(choices)
export(complete)
export(connectivity)
export(decode)
export(freq)
export(group)
export(grouped_rankings)
export(itempar)
export(pladmm)
export(plfit)
export(pltree)
export(qvcalc)
export(rankings)
export(read.soc)
export(read.soi)
export(read.toc)
export(read.toi)
export(str)
importFrom(CVXR,Minimize)
importFrom(CVXR,Problem)
importFrom(CVXR,Variable)
importFrom(CVXR,psolve)
importFrom(CVXR,quad_form)
importFrom(CVXR,value)
importFrom(Matrix,crossprod)
importFrom(Matrix,lu)
importFrom(Matrix,sparseMatrix)
importFrom(R6,R6Class)
importFrom(RSpectra,eigs)
importFrom(igraph,as_adj)
importFrom(igraph,components)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,graph_from_edgelist)
importFrom(igraph,simplify)
importFrom(matrixStats,rowCumsums)
importFrom(methods,cbind2)
importFrom(partykit,info_node)
importFrom(partykit,mob_control)
importFrom(partykit,nodeapply)
importFrom(partykit,nodeids)
importFrom(partykit,refit.modelparty)
importFrom(psychotools,itempar)
importFrom(psychotree,node_btplot)
importFrom(qvcalc,qvcalc)
importFrom(sandwich,estfun)
importFrom(stats,.getXlevels)
importFrom(stats,AIC)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,deviance)
importFrom(stats,df.residual)
importFrom(stats,formula)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,model.weights)
importFrom(stats,na.exclude)
importFrom(stats,na.omit)
importFrom(stats,optim)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,rexp)
importFrom(stats,runif)
importFrom(stats,simulate)
importFrom(stats,terms)
importFrom(stats,vcov)
importFrom(stats,xtabs)
importFrom(utils,combn)
importFrom(utils,read.csv)
importFrom(utils,str)
importFrom(utils,tail)
